function y = EDFA(Optical_input,AmplifierModelType,LockedTarget,Unsat_GaindB,Sat_OutputdBm,IncludeNoise,NFdB)
%
% Usage: y = EDFA(Optical_input,AmplifierModelType,LockedTarget,Unsat_GaindB,Sat_OutputdBm,IncludeNoise,NFdB)
%
% Purpose = This module is to simulate an ideal EDFA (gain and noise figure are wavelength independent).
%
% Sub_modules = EDFA_gain, EDFA_power, EDFA_saturation 
%             = all of them belong to EDFA 
%
% Input(Optical_input) = Optical signal
% Output(y) = Amplified optical signal
%
%=== Physical Parameters=======
%
% AmplifierModelType = 'Gain', 'Power', or 'Saturation' 
%                    = 'Gain' (default)
% LockedTarget = Target value(Gain[dB] or OutputPower[dBm] depending on AmplifierModelType, EDFA_gain or EDFA_power )  
%              = 20 (default)
% Unsat_GaindB = Unsaturated gain in [dB] (used for EDFA_saturation)
%              = 20 (default)
% Sat_OutputdBm = Optical power[dBm] at the 3dB optical gain compression (used for EDFA_saturation).
%              = 20 (default)
% IncludeNoise = 'Yes' or 'No' 
%              = 'Yes' (default)
% NFdB = Noise figure in [dB]
%      = 5 (default)
%
% Note1) ASE noise modeled by a complex Gaussain process (in time domain)
% Note2) Single polarization assumed.
%
%----------------------------------------------------------
% $ Revision: 1.1 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
%====================================================================================

if nargin < 7
    NFdB = 5; %[dB]
end
if nargin < 6
   IncludeNoise = 'Yes';
end
if nargin < 5
   Sat_OutputdBm = 20;%[dBm]
end
if nargin < 4
   Unsat_GaindB = 20;%[dB]
end
if nargin <3
    LockedTarget = 20; % Gain [dB]
end
if nargin<2
    AmplifierModelType = 'Gain';
end



%== Amplifier Model Selection ==

if strcmp(AmplifierModelType, 'Gain') == 1

	y = EDFA_gain(Optical_input,LockedTarget,IncludeNoise,NFdB);

elseif strcmp(AmplifierModelType, 'Power') == 1

	y = EDFA_power(Optical_input,LockedTarget,IncludeNoise,NFdB);

elseif strcmp(AmplifierModelType, 'Saturation') == 1
    
	y = EDFA_saturation(Optical_input,Unsat_GaindB,Sat_OutputdBm,IncludeNoise,NFdB);
    
else
    error('AmplifierModelType = Gain, Power, or Saturation. Please check your spelling.')
    
end