function y = PhotoDiode_PIN(E_optical,Responsivity,ThermalNoiseDen,DarkCurrent,ShotNoiseFactor)  
%
% Usage : y = PhotoDiode_PIN(E_optical,Responsivity,ThermalNoiseDen,DarkCurrent,ShotNoiseFactor)
%
% Purpose = This module acts as a PIN photodiode with additive Gaussian noise.
%
% Input Signal (E_optical) = Received Optical Signal 
% Output Signal (y) = Photo Detected Electrical Signal 
%
% === Physical Parameters with default values ===
% Responsivity = 1.0 [A/W], [0,2]
% ThermalNoiseDen = 1e-22 [A^2/Hz], Power Spectral Density of Thermal noise 
% DarkCurrent = 0 [A], [0,10e-6]
% ShotNoiseFactor = 1   (1 if shot noise included, 0 if not)
%
% (global variables) DataLength SampleRate 
%
% Note) Shot Noise is modeled with Gaussian distribution.
%
%------------------------------------------------------------
% $ Revision: 1.0 $ Date: 2011/02/22 by Jong-Hyung Lee  
%------------------------------------------------------------
%
%====================================================================================
global DataLength SampleRate 

if nargin<5,
   ShotNoiseFactor = 1;
end
if nargin<4,
  DarkCurrent = 0 ;
end
if nargin<3,
  ThermalNoiseDen = 1e-22 ;% [A^2/Hz]
end
if nargin<2, 
  Responsivity = 1.0; %[A/W]
end


%======== PhotoDetection Process =============
P_received = E_optical.*conj(E_optical);
i_gen = Responsivity*P_received;

i_gen_ave = Responsivity*sum(P_received)/length(P_received);

%==== NoiseGeneration ====
q = 1.602e-19; %[C]=[A sec]=[A/Hz] electron charge

n_shot = 2*q*(DarkCurrent+i_gen_ave);
i_nshot = ShotNoiseFactor*sqrt(n_shot*SampleRate)*randn(1,DataLength);
i_nth = sqrt(ThermalNoiseDen*SampleRate)*randn(1,DataLength);

%======= Total output current of PIN ======

y = i_gen + DarkCurrent + i_nth + i_nshot;
